/*-------------------------------------------------------------------------+
| Copyright 2013 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.compose;

import org.fortiss.tooling.base.dnd.ConnectionDragContext;
import org.fortiss.tooling.base.model.base.EntryConnectorBase;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.kernel.extension.data.IConnectionCompositionContext;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Base class for all connector - hierarchical element connections
 * 
 * @author igna
 */
public abstract class ConnectorHierarchicElementConnectionCompositorBase<HE extends IHierarchicElement, S extends IConnector, T extends IHierarchicElement>
		extends ConnectionCompositorBase<HE, S, T> {
	/** {@inheritDoc} */
	@Override
	public boolean canConnect(S source, T target, HE parent,
			IConnectionCompositionContext context) {
		if(context instanceof ConnectionDragContext) {
			ConnectionDragContext dc = (ConnectionDragContext)context;
			if(dc.isSourceFreeConnector() && dc.isTargetRoot()) {
				return canConnectInternally(source, target);
			}
		}
		return true;
	}

	/**
	 * Creates the application specific source {@link IConnector} model element.
	 * 
	 * @param otherConnector
	 *            The opposite connector the new entry connector will be connected to.
	 * 
	 * @return The newly created entry {@link IConnector}.
	 */
	protected abstract IConnector createEntryConnector(S otherConnector);

	/**
	 * Creates the application specific target {@link IConnector} model element.
	 * 
	 * @param otherConnector
	 *            The opposite connector the new exit connector will be connected to.
	 * 
	 * @return The newly created exit {@link IConnector}.
	 */
	protected abstract IConnector createExitConnector(S otherConnector);

	/** {@inheritDoc} */
	@Override
	public boolean connect(S source, T target, HE parent, IConnectionCompositionContext context) {
		boolean isSrcEntry = source instanceof EntryConnectorBase;
		isSrcEntry = sameParent(target, source) ? isSrcEntry : !isSrcEntry;
		IConnector conn = isSrcEntry ? createEntryConnector(source) : createExitConnector(source);
		avoidDuplicateConnectorName(target, conn);

		createNewTargetDropContext(target, context, conn);

		ITopLevelElement topElement =
				IPersistencyService.getInstance().getTopLevelElementFor(parent);
		createConnection(source, conn, parent, topElement);
		return true;
	}
}
