/*-------------------------------------------------------------------------+
| Copyright 2024 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.quality.data;

import java.time.LocalDateTime;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;

/**
 * Storage element corresponding to a {@Link IProjectRootElement}. It stores the last refresh time
 * as {@link LocalDateTime}, its respective {@link ITopLevelElement} and the {@link MetricTreeNode}
 * storing the data.
 * 
 * @author groh
 */
public class DataRootElement {
	/** The last time a metric tree was refreshed. */
	private int commitTime;

	/** The last time the metric tree was refreshed. */
	private EObject topLevelElement;

	/** Root {@link MetricTreeNode} for the metric tree containing all data. */
	private MetricTreeNode rootNode;

	/** Constructor. */
	public DataRootElement(int commitTime, EObject topLevelElement, MetricTreeNode rootNode) {
		super();
		this.commitTime = commitTime;
		this.topLevelElement = topLevelElement;
		this.rootNode = rootNode;
	}

	/** Getter for {@link #commitTime}. */
	public int getCommitTime() {
		return commitTime;
	}

	/** Setter for {@link #commitTime}. */
	public void setCommitTime(int commitTime) {
		this.commitTime = commitTime;
	}

	/** Getter for {@link #topLevelElement}. */
	public EObject getTopLevelElement() {
		return topLevelElement;
	}

	/** Setter for {@link #topLevelElement}. */
	public void setTopLevelElement(EObject topLevelElement) {
		this.topLevelElement = topLevelElement;
	}

	/** Getter for {@link #rootNode}. */
	public MetricTreeNode getRootNode() {
		return rootNode;
	}

	/** Setter for {@link #rootNode}. */
	public void setRootNode(MetricTreeNode metricTreeNode) {
		this.rootNode = metricTreeNode;
	}
}
